﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ImageDataSample
{
    public partial class Form1 : Form
    {

        /// <summary>
        /// 表示用ImageDataクラスオブジェクト
        /// </summary>
        ImagingSolution.Imaging.ImageData _img;

        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// 画像ファイルを開く
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            //ファイルを開くダイアログボックスの作成  
            var ofd = new OpenFileDialog();
            //ファイルフィルタ  
            ofd.Filter = "Image File(*.bmp,*.jpg,*.png,*.tif)|*.bmp;*.jpg;*.png;*.tif|Bitmap(*.bmp)|*.bmp|Jpeg(*.jpg)|*.jpg|PNG(*.png)|*.png";
            //ダイアログの表示 （Cancelボタンがクリックされた場合は何もしない）
            if (ofd.ShowDialog() == DialogResult.Cancel) return;

            if (_img != null) _img.Dispose();

            _img = new ImagingSolution.Imaging.ImageData(ofd.FileName);

            picImage.Image = _img.ToBitmap();
        }

        /// <summary>
        /// 名前を付けて画像を保存
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuFileSave_Click(object sender, EventArgs e)
        {
            // 画像データが設定されていない場合は何もしない
            if (_img == null) return;

            //名前を付けて保存ダイアログボックスの作成  
            var sfd = new SaveFileDialog();
            //ファイルフィルタ  
            sfd.Filter = "Bitmap(*.bmp)|*.bmp|Jpeg(*.jpg)|*.jpg|PNG(*.png)|*.png|CSV(*.csv)|*.csv";
            //ダイアログの表示 （Cancelボタンがクリックされた場合は何もしない）
            if (sfd.ShowDialog() == DialogResult.Cancel) return;

            // 名前を付けて画像データを保存
            _img.Save(sfd.FileName);
        }

        /// <summary>
        /// 終了
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// 輝度値の表示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void picImage_MouseMove(object sender, MouseEventArgs e)
        {
            if (_img == null) return;

            // X座標
            int px = e.X;
            // Y座標
            int py = e.Y;

            if (_img.Channel == 1)
            {
                // モノクロの場合
                lblBright.Text = "(" + px.ToString() + ", " + py.ToString() + ") = "
                    + _img[py, px].ToString();

            }
            else if (_img.Channel == 3)
            {
                // カラーの場合
                lblBright.Text = "(" + px.ToString() + ", " + py.ToString() + ") = ("
                    + _img[py, px, 2].ToString() + ", " // R
                    + _img[py, px, 1].ToString() + ", " // G
                    + _img[py, px, 0].ToString() + ")"; // B
            }

        }

        private void mnuFilter_Click(object sender, EventArgs e)
        {
            // フィルタ処理

            if (_img == null) return;

            var sw = new System.Diagnostics.Stopwatch();

            // 時間計測開始
            sw.Start();

            // 処理後のImageDataクラス
            ImagingSolution.Imaging.ImageData dst = null;

            // クリックされたメニュー
            var mnu = sender as System.Windows.Forms.ToolStripMenuItem;

            switch (mnu.Text){

                case "GrayScale":
                    // グレースケール変換
                    ConvertGrayScale(_img, ref dst);

                    break;

                case "Gaussian":
                    // ガウシアンフィルタ
                    GaussianBlur(_img, ref dst);
                    break;

                case "Sobel":
                    // ソーベルフィルタ
                    Sobel(_img, ref dst);
                    break;
            }

            // 時間計測停止
            sw.Stop();

            // 処理時間の表示
            lblTime.Text = sw.ElapsedMilliseconds.ToString() + "mSec";

            // 元の画像データを解放
            _img.Dispose();

            // 表示用画像データを設定
            _img = dst;

            // 画像の表示
            picImage.Image = _img.ToBitmap();

        }

        /// <summary>
        /// グレースケールへ変換
        /// </summary>
        private void ConvertGrayScale(ImagingSolution.Imaging.ImageData src, ref ImagingSolution.Imaging.ImageData dst)
        {
            int i, j;
            int width = src.Width;
            int height = src.Height;

            // dstが無効の場合、モノクロのImageDataクラスをインスタンス
            if (dst == null) dst = new ImagingSolution.Imaging.ImageData(width, height, System.Drawing.Imaging.PixelFormat.Format8bppIndexed);

            for (j = 0; j < height; j++)
            {
                for (i = 0; i < width; i++)
                {
                    dst[j, i] = (int)(
                        src[j, i, 2] * 0.299   // R 
                      + src[j, i, 1] * 0.587   // G
                      + src[j, i, 0] * 0.114); // B
                }
            }
        }

        /// <summary>
        /// ガウシアンフィルタ
        /// </summary>
        /// <param name="src"></param>
        /// <param name="dst"></param>
        private void GaussianBlur(ImagingSolution.Imaging.ImageData src, ref ImagingSolution.Imaging.ImageData dst)
        {
            int i, j;
            int width = src.Width;
            int height = src.Height;
            int channel = src.Channel;

            // dstが無効の場合、ImageDataクラスオブジェクトのクローンを作成
            if (dst == null) dst = src.Clone(false);

            for (j = 0; j < height; j++)
            {
                for (i = 0; i < width * channel; i++)
                {
                    dst[j, i] = (
                        src[j - 1, i - channel] + src[j - 1, i] * 2 + src[j - 1, i + channel]
                        + src[j, i - channel] * 2 + src[j, i] * 4 + src[j, i + channel] * 2
                        + src[j + 1, i - channel] + src[j + 1, i] * 2 + src[j + 1, i + channel]
                        ) / 16;
                }
            }
        }

        /// <summary>
        /// ソーベルフィルタ
        /// </summary>
        /// <param name="src"></param>
        /// <param name="dst"></param>
        private void Sobel(ImagingSolution.Imaging.ImageData src, ref ImagingSolution.Imaging.ImageData dst)
        {
            int i, j;
            int width = src.Width;
            int height = src.Height;
            int channel = src.Channel;

            // 処理後画像データを確保(メモリのコピーはしない)
            var XSobel = src.Clone(false);
            var YSobel = src.Clone(false);

            // 負の値は絶対値にする
            XSobel.MinusValueMode = ImagingSolution.Imaging.ImageData.MinusValueModeEnum.Absolute;
            YSobel.MinusValueMode = ImagingSolution.Imaging.ImageData.MinusValueModeEnum.Absolute;

            for (j = 0; j < height; j++)
            {
                for (i = 0; i < width * channel; i++)
                {
                    XSobel[j, i] = (
                        -src[j - 1, i - channel] + src[j - 1, i + channel]
                        - src[j, i - channel] * 2 + src[j, i + channel] * 2
                        - src[j + 1, i - channel] + src[j + 1, i + channel]
                        );
                    YSobel[j, i] = (
                        -src[j - 1, i - channel] - src[j - 1, i] * 2 - src[j - 1, i + channel]
                        + src[j + 1, i - channel] + src[j + 1, i] * 2 + src[j + 1, i + channel]
                        );
                }
            }
            // X方向とY方向を足し合わせる
            dst = XSobel + YSobel;

            // 解放
            XSobel.Dispose();
            YSobel.Dispose();
        }
    }
}
